import React, { useState, useCallback, useEffect, useMemo, Children, isValidElement, createContext, useContext, forwardRef, useId } from 'react';

const useWindowWidth = (isResponsive = true) => {
    const [windowWidth, setWindowSize] = useState(window.innerWidth);
    const handleResize = useCallback(() => {
        setWindowSize(window.innerWidth);
    }, []);
    useEffect(() => {
        if (isResponsive) {
            window.addEventListener('resize', handleResize);
        }
        else {
            window.removeEventListener('resize', handleResize);
        }
        return () => {
            window.removeEventListener('resize', handleResize);
        };
    }, [isResponsive, handleResize]);
    return windowWidth;
};

const findBreakpoint = (breakpoints, windowWidth) => {
    const sortedBreakPoints = Object.keys(breakpoints)
        .map(Number)
        .sort((a, b) => a - b);
    let bp = null;
    for (const breakPoint of sortedBreakPoints) {
        if (windowWidth > breakPoint) {
            bp = breakPoint;
        }
    }
    return bp !== null && bp !== void 0 ? bp : sortedBreakPoints[0];
};

const DEFAULT_COLUMNS = 3;
const useColumnsCount = (columns) => {
    const isResponsive = typeof columns === 'object';
    const windowWidth = useWindowWidth(isResponsive);
    const columnsCount = useMemo(() => {
        var _a;
        if (!isResponsive) {
            return columns !== null && columns !== void 0 ? columns : DEFAULT_COLUMNS;
        }
        const breakPoint = findBreakpoint(columns, windowWidth);
        return (_a = columns[breakPoint]) !== null && _a !== void 0 ? _a : DEFAULT_COLUMNS;
    }, [isResponsive, windowWidth, columns]);
    return columnsCount;
};

const createEmptyColumns = (count) => {
    return Array.from({ length: count }, () => []);
};
const useMasonry = (children, columns) => {
    const noOfColumns = useColumnsCount(columns);
    const columnsChildren = useMemo(() => {
        const group = createEmptyColumns(noOfColumns);
        Children.forEach(children, (child, index) => {
            if (isValidElement(child)) {
                group[index % noOfColumns].push(child);
            }
        });
        return group.filter((column) => column.length !== 0);
    }, [noOfColumns, children]);
    return columnsChildren;
};

const MasonryItemContext = createContext({
    column: 0,
    position: 0,
});
const useMasonryItem = () => useContext(MasonryItemContext);

const MasonryBase = (props, forwaredRef) => {
    const { gap, as: Component = 'div', columnProps, columns, ...rest } = props;
    const uniq = useId();
    const columnsChildren = useMasonry(props.children, columns);
    return (React.createElement(Component, { style: { display: 'flex', gap }, ref: forwaredRef, ...rest }, columnsChildren.map((column, index) => {
        return (React.createElement(Component, { key: `Masonry__Column_${uniq}_${index}`, "data-masonry-column": index + 1, style: {
                display: 'flex',
                flex: 1,
                flexDirection: 'column',
                gap,
            }, ...columnProps }, column.map((child, childIndex) => {
            return (React.createElement(MasonryItemContext.Provider, { value: { column: index, position: childIndex }, key: `Masonry__Column_Child_${uniq}_${childIndex}` }, child));
        })));
    })));
};
const Masonry = forwardRef(MasonryBase);

export { Masonry as default, useMasonry, useMasonryItem };
//# sourceMappingURL=react-layout-masonry.js.map
